@@ -203,7 +203,7 @@ static gps_mask_t geostar_analyze(struct gps_device_t *session)
 		mask |= STATUS_SET;
 	    }
 	}
-	mask |= TIME_SET | PPSTIME_IS | LATLON_SET | ALTITUDE_SET | SPEED_SET | TRACK_SET | DOP_SET | USED_IS | REPORT_IS;
+	mask |= TIME_SET | NTPTIME_IS | LATLON_SET | ALTITUDE_SET | SPEED_SET | TRACK_SET | DOP_SET | USED_IS | REPORT_IS;
 
 	gpsd_log(&session->context->errout, LOG_INF,
 		 "Geographic coordinates %f %g %g %g %g %g\n",
@@ -435,8 +435,10 @@ static gps_mask_t geostar_analyze(struct gps_device_t *session)
 	ul1 = getleu32(buf, OFFSET(2));
 	ul2 = getleu32(buf, OFFSET(3));
 	ul3 = getleu32(buf, OFFSET(4));
-	(void)snprintf(session->subtype, sizeof(session->subtype), "%d.%d %d.%d.%d %x %c-%d\n",
-		ul4>>16, ul4&0xFFFF, ul1>>9, (ul1>>5)&0xF, ul1&0x1F, ul2, ul3>>24, ul3&0x00FFFFFF);
+        (void)snprintf(session->subtype, sizeof(session->subtype),
+                "%u.%u %u.%u.%u %x %c-%u\n",
+                ul4>>16, ul4&0xFFFF, ul1>>9, (ul1>>5)&0xF, ul1&0x1F, ul2,
+                ul3>>24, ul3&0x00FFFFFF);
 	gpsd_log(&session->context->errout, LOG_INF,
 		 "Response to Request FW version command: %s\n",
 		 session->subtype);
@@ -546,6 +548,7 @@ static bool geostar_speed_switch(struct gps_device_t *session,
 			      speed_t speed, char parity, int stopbits)
 {
     unsigned char buf[4 * 4];
+    int iparity;
 
     switch (parity) {
     case 'E':
@@ -562,11 +565,12 @@ static bool geostar_speed_switch(struct gps_device_t *session,
 	parity = (char)0;
 	break;
     }
+    iparity = parity;
 
     putbe32(buf, 0, session->driver.geostar.physical_port);
     putbe32(buf, 4, speed);
     putbe32(buf, 8, stopbits);
-    putbe32(buf, 12, parity);
+    putbe32(buf, 12, iparity);
     (void)geostar_write(session, 0x41, buf, 4);
 
     return true;	/* it would be nice to error-check this */
