@@ -2,6 +2,7 @@
  * Handle the Trimble TSIP packet format
  * by Rob Janssen, PE1CHL.
  * Accutime Gold support by Igor Socec <igorsocec@gmail.com>
+ * Trimble RES multi-constelation support by Nuno Goncalves <nunojpg@gmail.com>
  *
  * Week counters are not limited to 10 bits. It's unknown what
  * the firmware is doing to disambiguate them, if anything; it might just
@@ -11,6 +12,17 @@
  * This file is Copyright (c) 2010 by the GPSD project
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
+
+#ifdef __linux__
+/* FreeBSD chokes on this */
+/* if we insist on C99, then we need this to get M_LN2 from math.h */
+/* 500 mean X/Open 1995 */
+#define _XOPEN_SOURCE 500
+/* round() needs  _POSIX_C_SOURCE >= 200112L */
+#define  _POSIX_C_SOURCE 200112L
+#endif /* __linux__ */
+
+
 #include <sys/time.h>		/* for select() */
 #include <string.h>
 #include <stdio.h>
@@ -33,7 +45,7 @@ void configuration_packets_accutime_gold(struct gps_device_t *session);
 void configuration_packets_generic(struct gps_device_t *session);
 
 #ifdef TSIP_ENABLE
-#define TSIP_CHANNELS	12
+#define TSIP_CHANNELS	15
 
 static int tsip_write(struct gps_device_t *session,
 		      unsigned int id, unsigned char *buf, size_t len)
@@ -119,7 +131,7 @@ static gps_mask_t tsip_parse_input(struct gps_device_t *session)
     int i, j, len, count;
     gps_mask_t mask = 0;
     unsigned int id;
-    uint8_t u1, u2, u3, u4, u5;
+    uint8_t u1, u2, u3, u4, u5, u6, u7, u8, u9, u10;
     int16_t s1, s2, s3, s4;
     int32_t sl1, sl2, sl3;
     uint32_t ul1, ul2;
@@ -214,6 +226,7 @@ static gps_mask_t tsip_parse_input(struct gps_device_t *session)
 		u4 = getub(buf, 6); /* Build hour */
 		s2 = (int16_t)getbeu16(buf, 10); /* Hardware Code */
 		u5 = getub(buf, 12); /* Length of Hardware ID */
+		/* coverity_submit[tainted_data] */
 		for (i=0; i < (int)u5; i++) {
 		    buf2[i] = (char)getub(buf, 13+i); /* Hardware ID in ASCII */
 		}
@@ -253,7 +266,7 @@ static gps_mask_t tsip_parse_input(struct gps_device_t *session)
 	    session->context->valid |= LEAP_SECOND_VALID;
 	    session->newdata.time =
 		gpsd_gpstime_resolve(session, (unsigned short)s1, (double)f1);
-	    mask |= TIME_SET | PPSTIME_IS;
+	    mask |= TIME_SET | NTPTIME_IS;
 	}
 	gpsd_log(&session->context->errout, LOG_INF,
 		 "GPS Time %f %d %f\n", f1, s1, f2);
@@ -358,7 +371,7 @@ static gps_mask_t tsip_parse_input(struct gps_device_t *session)
 		gpsd_gpstime_resolve(session,
 				  (unsigned short)session->context->gps_week,
 				  (double)f2);
-	    mask |= TIME_SET | PPSTIME_IS;
+	    mask |= TIME_SET | NTPTIME_IS;
 	}
 	mask |= LATLON_SET | ALTITUDE_SET | CLEAR_IS | REPORT_IS;
 	gpsd_log(&session->context->errout, LOG_DATA,
@@ -511,8 +524,114 @@ static gps_mask_t tsip_parse_input(struct gps_device_t *session)
 		session->gpsdata.satellites_visible = i;
 	}
 	break;
+     case 0x5d:
+        /* GNSS Satellite Tracking Status (multi-GNSS operation) */
+	if (len != 26)
+	    break;
+	u1 = getub(buf, 0);	/* PRN */
+	u2 = getub(buf, 1);	/* chan */
+	u3 = getub(buf, 2);	/* Acquisition flag */
+	u4 = getub(buf, 3);	/* SV used in Position or Time calculation*/
+	f1 = getbef32((char *)buf, 4);	/* Signal level */
+	f2 = getbef32((char *)buf, 8);	/* time of Last measurement */
+	d1 = getbef32((char *)buf, 12) * RAD_2_DEG;	/* Elevation */
+	d2 = getbef32((char *)buf, 16) * RAD_2_DEG;	/* Azimuth */
+	u5 = getub(buf, 20);	/* old measurement flag */
+	u6 = getub(buf, 21);	/* integer msec flag */
+	u7 = getub(buf, 22);	/* bad data flag */
+	u8 = getub(buf, 23);	/* data collection flag */
+	u9 = getub(buf, 24);	/* Used flags */
+	u10 = getub(buf, 25);	/* SV Type */
+
+	i = u2;			/* channel number */
+	gpsd_log(&session->context->errout, LOG_INF,
+		"Satellite Tracking Status: Ch %2d Con %d PRN %3d Acq %d "
+		"Use %d SNR %4.1f LMT %.04f El %4.1f Az %5.1f Old %d Int %d "
+	        "Bad %d Col %d TPF %d SVT %d\n",
+		i, u10, u1, u3, u4, f1, f2, d1, d2, u5, u6, u7, u8, u9, u10);
+	if (i < TSIP_CHANNELS) {
+	    if (d1 >= 0.0) {
+		session->gpsdata.skyview[i].PRN = (short)u1;
+		session->gpsdata.skyview[i].ss = (double)f1;
+		session->gpsdata.skyview[i].elevation = (short)round(d1);
+		session->gpsdata.skyview[i].azimuth = (short)round(d2);
+		session->gpsdata.skyview[i].used = (bool)u4;
+	    } else {
+		session->gpsdata.skyview[i].PRN = (short)u1;
+		session->gpsdata.skyview[i].elevation =
+		    session->gpsdata.skyview[i].azimuth = 0;
+		session->gpsdata.skyview[i].ss = 0.0;
+		session->gpsdata.skyview[i].used = false;
+	    }
+	    if (++i == session->gpsdata.satellites_visible) {
+		session->gpsdata.skyview_time = NAN;
+		mask |= SATELLITE_SET;	/* last of the series */
+	    }
+	    if (i > session->gpsdata.satellites_visible)
+		session->gpsdata.satellites_visible = i;
+	}
+	break;
     case 0x5e:			/* Additional Fix Status Report */
 	break;
+    case 0x6c:			/* Satellite Selection List */
+	//u1 = getub(buf, 0);	/* nsvs/dimension UNUSED */
+	count = (int)getub(buf, 17);
+	if (len != (18 + count))
+	    break;
+	session->driver.tsip.last_6d = now;	/* keep timestamp for request */
+#ifdef __UNUSED__
+	/*
+	 * This looks right, but it sets a spurious mode value when
+	 * the satellite constellation looks good to the chip but no
+	 * actual fix has yet been acquired.  We should set the mode
+	 * field (which controls gpsd's fix reporting) only from sentences
+	 * that convey actual fix information, like 0x20, otherwise we
+	 * get results like triggering their error modeler spuriously.
+	 */
+	switch (u1 & 7) {	/* dimension */
+	case 3:
+	    // session->gpsdata.status = STATUS_FIX;
+	    session->newdata.mode = MODE_2D;
+	    break;
+	case 4:
+	    // session->gpsdata.status = STATUS_FIX;
+	    session->newdata.mode = MODE_3D;
+	    break;
+	default:
+	    // session->gpsdata.status = STATUS_NO_FIX;
+	    session->newdata.mode = MODE_NO_FIX;
+	    break;
+	}
+	mask |= MODE_SET;
+#endif /* __UNUSED__ */
+	session->gpsdata.satellites_used = count;
+	session->gpsdata.dop.pdop = getbef32((char *)buf, 1);
+	session->gpsdata.dop.hdop = getbef32((char *)buf, 5);
+	session->gpsdata.dop.vdop = getbef32((char *)buf, 9);
+	session->gpsdata.dop.tdop = getbef32((char *)buf, 13);
+	session->gpsdata.dop.gdop =
+	    sqrt(pow(session->gpsdata.dop.pdop, 2) +
+		 pow(session->gpsdata.dop.tdop, 2));
+
+	memset(session->driver.tsip.sats_used, 0,
+		sizeof(session->driver.tsip.sats_used));
+	buf2[0] = '\0';
+	for (i = 0; i < count; i++)
+	    str_appendf(buf2, sizeof(buf2),
+			   " %d", session->driver.tsip.sats_used[i] =
+			   (short)getub(buf, 18 + i));
+	gpsd_log(&session->context->errout, LOG_DATA,
+		 "AIVSS: 0x6d status=%d used=%d "
+		 "pdop=%.1f hdop=%.1f vdop=%.1f tdop=%.1f gdup=%.1f\n",
+		 session->gpsdata.status,
+		 session->gpsdata.satellites_used,
+		 session->gpsdata.dop.pdop,
+		 session->gpsdata.dop.hdop,
+		 session->gpsdata.dop.vdop,
+		 session->gpsdata.dop.tdop,
+		 session->gpsdata.dop.gdop);
+	mask |= DOP_SET | STATUS_SET | USED_IS;
+	break;
     case 0x6d:			/* All-In-View Satellite Selection */
 	u1 = getub(buf, 0);	/* nsvs/dimension */
 	count = (int)((u1 >> 4) & 0x0f);
@@ -620,7 +739,7 @@ static gps_mask_t tsip_parse_input(struct gps_device_t *session)
 		gpsd_gpstime_resolve(session,
 				  (unsigned short)session->context->gps_week,
 				  (double)f1);
-	    mask |= TIME_SET | PPSTIME_IS;
+	    mask |= TIME_SET | NTPTIME_IS;
 	}
 	gpsd_log(&session->context->errout, LOG_INF,
 		 "GPS DP LLA %f %f %f %f\n",
@@ -712,7 +831,7 @@ static gps_mask_t tsip_parse_input(struct gps_device_t *session)
 						      (unsigned short)s4,
 						      (double)ul1 *1e-3);
 	    mask |=
-		TIME_SET | PPSTIME_IS | LATLON_SET | ALTITUDE_SET | SPEED_SET |
+		TIME_SET | NTPTIME_IS | LATLON_SET | ALTITUDE_SET | SPEED_SET |
 		TRACK_SET | CLIMB_SET | STATUS_SET | MODE_SET | CLEAR_IS |
 		REPORT_IS;
 	    gpsd_log(&session->context->errout, LOG_DATA,
@@ -749,7 +868,7 @@ static gps_mask_t tsip_parse_input(struct gps_device_t *session)
 	    }
 	    session->newdata.time =
 		gpsd_gpstime_resolve(session,
-				  (unsigned short)s1, (double)ul1 *1e3);
+				  (unsigned short)s1, (double)ul1 *1e-3);
 	    session->gpsdata.status = STATUS_NO_FIX;
 	    session->newdata.mode = MODE_NO_FIX;
 	    if ((u2 & 0x01) == (uint8_t) 0) {	/* Fix Available */
@@ -782,7 +901,7 @@ static gps_mask_t tsip_parse_input(struct gps_device_t *session)
 	    if ((session->newdata.track = atan2(d1, d2) * RAD_2_DEG) < 0)
 		session->newdata.track += 360.0;
 	    mask |=
-		TIME_SET | PPSTIME_IS | LATLON_SET | ALTITUDE_SET | SPEED_SET |
+		TIME_SET | NTPTIME_IS | LATLON_SET | ALTITUDE_SET | SPEED_SET |
 		TRACK_SET |CLIMB_SET | STATUS_SET | MODE_SET | CLEAR_IS |
 		REPORT_IS;
 	    gpsd_log(&session->context->errout, LOG_DATA,
@@ -795,6 +914,15 @@ static gps_mask_t tsip_parse_input(struct gps_device_t *session)
 		     session->newdata.mode, session->gpsdata.status);
 	    break;
 
+	case 0x4a:		/* Set PPS Characteristics */
+	    break;
+
+	case 0x4e:		/* PPS Output */
+	    break;
+
+	case 0xa2:		/* UTC/GPS Timing */
+	    break;
+
 	case 0xab:		/* Thunderbolt Timing Superpacket */
 	    if (len != 17) {
 		gpsd_log(&session->context->errout, 4, "pkt 0xab len=%d\n", len);
@@ -810,7 +938,7 @@ static gps_mask_t tsip_parse_input(struct gps_device_t *session)
 		session->context->valid |= LEAP_SECOND_VALID;
 		session->newdata.time =
 		    gpsd_gpstime_resolve(session, (unsigned short)s1, (double)ul1);
-		mask |= TIME_SET | PPSTIME_IS | CLEAR_IS;
+		mask |= TIME_SET | NTPTIME_IS | CLEAR_IS;
 		gpsd_log(&session->context->errout, LOG_DATA,
 			 "SP-TTS 0xab time=%.2f mask={TIME}\n",
 			 session->newdata.time);
@@ -947,6 +1075,7 @@ static gps_mask_t tsip_parse_input(struct gps_device_t *session)
     }
 
     if ((now - session->driver.tsip.last_5c) >= 5) {
+	/* 5d in multi-gnss devices */
 	/* Request Current Satellite Tracking Status */
 	putbyte(buf, 0, 0x00);	/* All satellites */
 	(void)tsip_write(session, 0x3c, buf, 1);
@@ -1006,7 +1135,7 @@ static void tsip_event_hook(struct gps_device_t *session, event_t event)
 	return;
     if (event == event_identified) {
 	unsigned char buf[100];
-	
+
 	/*
 	 * Set basic configuration, in case no hardware config response
 	 * comes back.
@@ -1027,7 +1156,7 @@ static void tsip_event_hook(struct gps_device_t *session, event_t event)
 	 */
 	session->driver.tsip.parity = session->gpsdata.dev.parity;
 	session->driver.tsip.stopbits =
-	    (uint) session->gpsdata.dev.stopbits;
+	    (unsigned int) session->gpsdata.dev.stopbits;
 	// gpsd_set_speed(session, session->gpsdata.dev.baudrate, 'O', 1);
     }
     if (event == event_deactivate) {
@@ -1126,8 +1255,18 @@ static void tsip_mode(struct gps_device_t *session, int mode)
 #ifdef TIMEHINT_ENABLE
 static double tsip_time_offset(struct gps_device_t *session UNUSED)
 {
-    /* FIX-ME: is a constant offset right here? */
-    return 0.075;
+    /*
+     * the offset depends on the generation and is not reliable, it often
+     * depends on the number of SV being tracked
+     * some example measurements:
+     * Trimble Resolution T   : 0.03s to 0.200s
+     * Trimble Resolution SMTx: 0.300 to 0.500s (unpredictable)
+     * Trimble RES SMT 360    : 0.028s (this appears very stable)
+     *
+     * Trimble users tend to be very picky, so we leave the offset to them.
+     * If we could detect the Trimble model maybe we could pick an offset.
+     */
+    return 0.0;
 }
 #endif /* TIMEHINT_ENABLE */
 
@@ -1135,14 +1274,14 @@ void configuration_packets_generic(struct gps_device_t *session)
 /* configure generic Trimble TSIP device to a known state */
 {
 	unsigned char buf[100];
-	
+
 	/* I/O Options */
 	putbyte(buf, 0, 0x1e);	/* Position: DP, MSL, LLA */
 	putbyte(buf, 1, 0x02);	/* Velocity: ENU */
 	putbyte(buf, 2, 0x00);	/* Time: GPS */
 	putbyte(buf, 3, 0x08);	/* Aux: dBHz */
 	(void)tsip_write(session, 0x35, buf, 4);
-	
+
 	/* Request Software Versions */
 	(void)tsip_write(session, 0x1f, NULL, 0);
 	/* Request Current Time */
