@@ -9,13 +9,23 @@
  * This file is Copyright (c) 2010 by the GPSD project
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
+
+#ifdef __linux__
+/* FreeBSD chokes on this */
+/* if we insist on C99, then we need this to get M_LN2 from math.h */
+/* if we insisnt on C99, then we need this to get M_LN2 from math.h */
+/* 500 means X/Open 1995 */
+#define _XOPEN_SOURCE 500
+/* round() needs  _POSIX_C_SOURCE >= 200112L */
+#define  _POSIX_C_SOURCE 200112L
+#endif /* __linux__ */
+
+
 #include <stdio.h>
 #include <stdbool.h>
 #include <string.h>
 #include <math.h>
-#ifndef S_SPLINT_S
 #include <unistd.h>
-#endif /* S_SPLINT_S */
 
 #include "gpsd.h"
 #include "bits.h"
@@ -58,7 +68,7 @@ static ssize_t end_write(int fd, void *d, size_t len)
 
     size_t n;
     for (n = 0; n < (size_t)(len/2); n++)
-	putle16(buf, n*2, data[n]); 
+	putle16(buf, n*2, data[n]);
     return write(fd, (char*)buf, len);
 }
 
@@ -145,13 +155,11 @@ static gps_mask_t handle1000(struct gps_device_t *session)
     /* ticks                      = getzlong(6); */
     /* sequence                   = getzword(8); */
     /* measurement_sequence       = getzword(9); */
-    /*@ -boolops -predboolothers @*/
     session->gpsdata.status = (getzword(10) & 0x1c) ? 0 : 1;
     if (session->gpsdata.status != 0)
 	session->newdata.mode = (getzword(10) & 1) ? MODE_2D : MODE_3D;
     else
 	session->newdata.mode = MODE_NO_FIX;
-    /*@ +boolops -predboolothers @*/
 
     /* solution_type                 = getzword(11); */
     session->gpsdata.satellites_used = (int)getzword(12);
@@ -167,10 +175,7 @@ static gps_mask_t handle1000(struct gps_device_t *session)
     unpacked_date.tm_sec = (int)getzword(24);
     unpacked_date.tm_isdst = 0;
     subseconds = (int)getzlong(25) / 1e9;
-    /*@ -compdef */
     session->newdata.time = (timestamp_t)mkgmtime(&unpacked_date) + subseconds;
-    /*@ +compdef */
-    /*@ -type @*/
     session->newdata.latitude = ((long)getzlong(27)) * RAD_2_DEG * 1e-8;
     session->newdata.longitude = ((long)getzlong(29)) * RAD_2_DEG * 1e-8;
     /*
@@ -179,7 +184,6 @@ static gps_mask_t handle1000(struct gps_device_t *session)
      * specify whether word 31 is geodetic or WGS 84.
      */
     session->newdata.altitude = ((long)getzlong(31)) * 1e-2;
-    /*@ +type @*/
     session->gpsdata.separation = ((short)getzword(33)) * 1e-2;
     session->newdata.altitude -= session->gpsdata.separation;
     session->newdata.speed = (int)getzlong(34) * 1e-2;
@@ -202,7 +206,7 @@ static gps_mask_t handle1000(struct gps_device_t *session)
     /* clock_drift_sd              = (int)getzlong(53) * 1e-2; */
 
     mask =
-	TIME_SET | PPSTIME_IS | LATLON_SET | ALTITUDE_SET | CLIMB_SET | SPEED_SET |
+	TIME_SET | NTPTIME_IS | LATLON_SET | ALTITUDE_SET | CLIMB_SET | SPEED_SET |
 	TRACK_SET | STATUS_SET | MODE_SET;
     gpsd_log(&session->context->errout, LOG_DATA,
 	     "1000: time=%.2f lat=%.2f lon=%.2f alt=%.2f track=%.2f speed=%.2f climb=%.2f mode=%d status=%d\n",
@@ -222,20 +226,16 @@ static gps_mask_t handle1002(struct gps_device_t *session)
     /* ticks                      = getzlong(6); */
     /* sequence                   = getzword(8); */
     /* measurement_sequence       = getzword(9); */
-    /*@+charint@*/
     int gps_week = getzword(10);
     int gps_seconds = getzlong(11);
     /* gps_nanoseconds            = getzlong(13); */
-    /*@-charint@*/
     /* Note: this week counter is not limited to 10 bits. */
     session->context->gps_week = (unsigned short)gps_week;
     session->gpsdata.satellites_used = 0;
     for (i = 0; i < ZODIAC_CHANNELS; i++) {
 	int status, prn;
-	/*@ -type @*/
 	session->driver.zodiac.Zv[i] = status = (int)getzword(15 + (3 * i));
 	session->driver.zodiac.Zs[i] = prn = (int)getzword(16 + (3 * i));
-	/*@ +type @*/
 
 	if (status & 1)
 	    session->gpsdata.satellites_used++;
@@ -353,9 +353,15 @@ static gps_mask_t zodiac_analyze(struct gps_device_t *session)
     unsigned int id =
 	(unsigned int)((session->lexer.outbuffer[3] << 8) |
 		       session->lexer.outbuffer[2]);
-    gpsd_log(&session->context->errout, LOG_RAW,
-	     "Raw Zodiac packet type %d length %zd: %s\n",
-	     id, session->lexer.outbuflen, gpsd_prettydump(session));
+    /*
+     * The guard looks superfluous, but it keeps the rather expensive
+     * gpsd_packetdump() function from being called even when the debug
+     * level does not actually require it.
+     */
+    if (session->context->errout.debug >= LOG_RAW)
+	gpsd_log(&session->context->errout, LOG_RAW,
+		 "Raw Zodiac packet type %d length %zd: %s\n",
+		 id, session->lexer.outbuflen, gpsd_prettydump(session));
 
     if (session->lexer.outbuflen < 10)
 	return 0;
@@ -391,7 +397,6 @@ static gps_mask_t zodiac_analyze(struct gps_device_t *session)
 static ssize_t zodiac_control_send(struct gps_device_t *session,
 				   char *msg, size_t len)
 {
-    /*@-usedef -compdef@*/
     unsigned short shortwords[256];
 
 #define min(x,y)	((x) < (y) ? x : y)
@@ -404,7 +409,6 @@ static ssize_t zodiac_control_send(struct gps_device_t *session,
     /* and if len isn't even, it's your own fault */
     return zodiac_spew(session, shortwords[0], shortwords + 1,
 		       (int)(len / 2 - 1));
-    /*@+usedef +compdef@*/
 }
 #endif /* CONTROLSEND_ENABLE */
 
