@@ -93,7 +93,10 @@ double wgs84_separation(double lat, double lon)
 void ecef_to_wgs84fix(struct gps_fix_t *fix, double *separation,
 		      double x, double y, double z,
 		      double vx, double vy, double vz)
-/* fill in WGS84 position/velocity fields from ECEF coordinates */
+/* fill in WGS84 position/velocity fields from ECEF coordinates
+ * x, y, z are all in meters
+ * vx, vy, vz are all in meters/second
+ */
 {
     double lambda, phi, p, theta, n, h, vnorth, veast, heading;
     const double a = WGS84A;	/* equatorial radius */
@@ -119,10 +122,23 @@ void ecef_to_wgs84fix(struct gps_fix_t *fix, double *separation,
 	-vx * sin(phi) * cos(lambda) - vy * sin(phi) * sin(lambda) +
 	vz * cos(phi);
     veast = -vx * sin(lambda) + vy * cos(lambda);
+
     fix->climb =
 	vx * cos(phi) * cos(lambda) + vy * cos(phi) * sin(lambda) +
 	vz * sin(phi);
+    /* sanity check the climb, 10,000 m/s should be a nice max */
+    if ( 9999.9 < fix->climb )
+	fix->climb = NAN;
+    else if ( -9999.9 > fix->speed )
+	fix->climb = NAN;
+
     fix->speed = sqrt(pow(vnorth, 2) + pow(veast, 2));
+    /* sanity check the speed, 10,000 m/s should be a nice max */
+    if ( 9999.9 < fix->speed )
+	fix->speed = NAN;
+    else if ( -9999.9 > fix->climb )
+	fix->speed = NAN;
+
     heading = atan2(fix_minuz(veast), fix_minuz(vnorth));
     if (heading < 0)
 	heading += 2 * GPS_PI;
