@@ -18,9 +18,7 @@ extern "C" {
 #include <time.h>
 #include <signal.h>
 #include <stdio.h>
-#ifndef S_SPLINT_S
 #include <pthread.h>	/* pacifies OpenBSD's compiler */
-#endif
 
 /*
  * 4.1 - Base version for initial JSON protocol (Dec 2009, release 2.90)
@@ -37,12 +35,12 @@ extern "C" {
  *       tag fields dropped from emitted JSON. The shape of the skyview
  *       structure has changed to make working with the satellites-used
  *       bits less confusing. (January 2015, release 3.12).
+ * 6.1 - Add navdata_t for more (nmea2000) info.
  */
 #define GPSD_API_MAJOR_VERSION	6	/* bump on incompatible changes */
-#define GPSD_API_MINOR_VERSION	0	/* bump on compatible changes */
+#define GPSD_API_MINOR_VERSION	1	/* bump on compatible changes */
 
 #define MAXCHANNELS	72	/* must be > 12 GPS + 12 GLONASS + 2 WAAS */
-#define GPS_PRNMAX	32	/* above this number are SBAS satellites */
 #define MAXUSERDEVS	4	/* max devices per user */
 #define GPS_PATH_MAX	128	/* for names like /dev/serial/by-id/... */
 
@@ -65,6 +63,17 @@ extern "C" {
  * timestamp_t is only microSec precision
  * timestamp_t and PPS do not play well together
  */
+
+/* we want cm accuracy and 0.0000001 degrees is 1.11 cm at the equator
+ * the equator is best case for longitude. At 45lat cut that in half.
+ * at 85lat make it 0.00000001
+ *
+ * this easily fits in a C double which has 15.95 digits of precision
+ * printf() format %f defaults to %.6f, which will truncate the result.
+ * so print with %.7f if you have a survey grade GPS.
+ *
+ * ref: https://en.wikipedia.org/wiki/Decimal_degrees
+ */
 typedef double timestamp_t;	/* Unix time in seconds with fractional part */
 
 struct gps_fix_t {
@@ -91,17 +100,21 @@ struct gps_fix_t {
 
 /*
  * Satellite ID classes.
- * IS-GPS-200 Revision E, paragraph 6.3.6
+ * According to IS-GPS-200 Revision H paragraph 6.3.6, and earlier revisions
+ * at least back to E, the upper bound of U.S. GPS PRNs is actually 64. However,
+ * NMEA0183 only allocates 1-32 for U.S. GPS IDs; it uses 33-64 for IDs ub the
+ * SBAS range.
  */
-#define GPS_PRN(n)	(((n) >= 1) && ((n) <= 63))	/* U.S. GPS satellite */
-#define GBAS_PRN(n)	((n) >= 64 && ((n) <= 119))	/* Ground Based Augmentation System and other augmentation systems */
-#define SBAS_PRN(n)	((n) >= 120 && ((n) <= 158))	/* Satellite Based Augmentation System */
-#define GNSS_PRN(n)	((n) >= 159 && ((n) <= 210))	/* other Global Navigation Satellite System */
+#define GPS_PRN(n)	(((n) >= 1) && ((n) <= 32))	/* U.S. GPS satellite */
+#define GBAS_PRN(n)	((n) >= 64 && ((n) <= 119))	/* Other GNSS (GLONASS) and Ground Based Augmentation System (eg WAAS)*/
+#define SBAS_PRN(n)	((n) >= 120 && ((n) <= 158))	/* Satellite Based Augmentation System (eg GAGAN)*/
+#define GNSS_PRN(n)	((n) >= 159 && ((n) <= 210))	/* other GNSS (eg BeiDou) */
 
 /*
  * GLONASS birds reuse GPS PRNs.
- * it is a GPSD convention to map them to IDs 65..96.
+ * It is an NMEA0183 convention to map them to pseudo-PRNs 65..96.
  * (some other programs push them to 33 and above).
+ * The US GPS constellation plans to use the 33-63 range.
  */
 #define GLONASS_PRN_OFFSET	64
 
@@ -138,14 +151,12 @@ struct gst_t {
 /* RTCM104 doesn't specify this, so give it the largest reasonable value */
 #define MAXHEALTH	(RTCM2_WORDS_MAX-2)
 
-#ifndef S_SPLINT_S
 /*
  * A nominally 30-bit word (24 bits of data, 6 bits of parity)
  * used both in the GPS downlink protocol described in IS-GPS-200
  * and in the format for DGPS corrections used in RTCM-104v2.
  */
-typedef /*@unsignedintegraltype@*/ uint32_t isgps30bits_t;
-#endif /* S_SPLINT_S */
+typedef uint32_t isgps30bits_t;
 
 /*
  * Values for "system" fields.  Note, the encoding logic is senstive to the
@@ -271,7 +282,7 @@ struct rtcm3_basic_rtk {
     unsigned int channel;	/* Satellite Frequency Channel Number
 				   (GLONASS only) */
     double pseudorange;		/* Pseudorange */
-    double rangediff;		/* PhaseRange – Pseudorange in meters */
+    double rangediff;		/* PhaseRange - Pseudorange in meters */
     unsigned char locktime;	/* Lock time Indicator */
 };
 
@@ -280,7 +291,7 @@ struct rtcm3_extended_rtk {
     unsigned int channel;	/* Satellite Frequency Channel Number
 				   (GLONASS only) */
     double pseudorange;		/* Pseudorange */
-    double rangediff;		/* PhaseRange – L1 Pseudorange */
+    double rangediff;		/* PhaseRange - L1 Pseudorange */
     unsigned char locktime;	/* Lock time Indicator */
     unsigned char ambiguity;	/* Integer Pseudorange
 					   Modulus Ambiguity */
@@ -522,7 +533,7 @@ struct rtcm3_t {
 	    char receiver[RTCM3_MAX_DESCRIPTOR+1];	/* Receiver string */
 	    char firmware[RTCM3_MAX_DESCRIPTOR+1];	/* Firmware string */
 	} rtcm3_1033;
-	char data[1024];		/* Max RTCM3 msg length is 1023 bytes */
+	unsigned char data[1024];	/* Max RTCM3 msg length is 1023 bytes */
     } rtcmtypes;
 };
 
@@ -830,11 +841,7 @@ struct subframe_t {
     };
 };
 
-#ifndef S_SPLINT_S
 typedef uint64_t gps_mask_t;
-#else
-typedef /*@unsignedintegraltype@*/ unsigned long long gps_mask_t;
-#endif /* S_SPLINT_S */
 
 /*
  * Is an MMSI number that of an auxiliary associated with a mother ship?
@@ -933,8 +940,10 @@ struct ais_t
 	struct {
 	    unsigned int ais_version;	/* AIS version level */
 	    unsigned int imo;		/* IMO identification */
+	    // cppcheck-suppress arrayIndexOutOfBounds
 	    char callsign[7+1];		/* callsign */
 #define AIS_SHIPNAME_MAXLEN	20
+	    // cppcheck-suppress arrayIndexOutOfBounds
 	    char shipname[AIS_SHIPNAME_MAXLEN+1];	/* vessel name */
 	    unsigned int shiptype;	/* ship type code */
 	    unsigned int to_bow;	/* dimension to bow */
@@ -963,6 +972,7 @@ struct ais_t
 #define AIS_TYPE6_BINARY_MAX	920	/* 920 bits */
 	    size_t bitcount;		/* bit count of the data */
 	    union {
+		// cppcheck-suppress arrayIndexOutOfBounds
 		char bitdata[(AIS_TYPE6_BINARY_MAX + 7) / 8];
 		/* Inland AIS - ETA at lock/bridge/terminal */
 		struct {
@@ -1233,6 +1243,7 @@ struct ais_t
 	    size_t bitcount;		/* bit count of the data */
 	    bool structured;		/* True match for DAC/FID? */
 	    union {
+		// cppcheck-suppress arrayIndexOutOfBounds
 		char bitdata[(AIS_TYPE8_BINARY_MAX + 7) / 8];
 		/* Inland static ship and voyage-related data */
 		struct {
@@ -1415,6 +1426,7 @@ struct ais_t
 			    unsigned int mmsi;
 			    unsigned int imo;
 #define DAC1FID17_ID_LENGTH		7
+			    // cppcheck-suppress arrayIndexOutOfBounds
 			    char callsign[DAC1FID17_ID_LENGTH+1];
 			    char other[DAC1FID17_ID_LENGTH+1];
 			} id;
@@ -1635,6 +1647,7 @@ struct ais_t
 	    unsigned int heading;	/* true heading */
 	    unsigned int second;	/* seconds of UTC timestamp */
 	    unsigned int regional;	/* regional reserved */
+	    // cppcheck-suppress arrayIndexOutOfBounds
 	    char shipname[AIS_SHIPNAME_MAXLEN+1];		/* ship name */
 	    unsigned int shiptype;	/* ship type code */
 	    unsigned int to_bow;	/* dimension to bow */
@@ -1790,10 +1803,11 @@ struct ais_t
     };
 };
 
+/* basic data, per PRN, from GPGSA and GPGSV */
 struct satellite_t {
     double ss;		/* signal-to-noise ratio (dB) */
-    bool used;		/* PRNs of satellites used in solution */
-    short PRN;		/* PRNs of satellite */
+    bool used;		/* this satellite used in solution */
+    short PRN;		/* PRN of this satellite */
     short elevation;	/* elevation of satellite */
     short azimuth;	/* azimuth */
 };
@@ -1823,6 +1837,25 @@ struct attitude_t {
     char yaw_st;
 };
 
+struct navdata_t {
+    unsigned int version;
+    double compass_heading;
+    double compass_deviation;
+    double compass_variation;
+    double air_temp;
+    double air_pressure;
+    double water_temp;
+    double depth;
+    double depth_offset;
+    double wind_speed;
+    double wind_dir;
+    double crosstrack_error;
+    unsigned int compass_status;
+    unsigned int log_cumulative;
+    unsigned int log_trip;
+    unsigned int crosstrack_status;
+};
+
 struct dop_t {
     /* Dilution of precision factors */
     double xdop, ydop, pdop, hdop, vdop, tdop, gdop;
@@ -1892,6 +1925,13 @@ struct timedelta_t {
 };
 #endif /* TIMEDELTA_DEFINED */
 
+struct oscillator_t {
+    bool running;			/* oscillator is running */
+    bool reference;			/* PPS reference is available */
+    bool disciplined;			/* oscillator is GPS-disciplined */
+    int delta;				/* last observed PPS delta */
+};
+
 /*
  * Someday we may support Windows, under which socket_t is a separate type.
  * In the meantime, having a typedef for this semantic kind is no bad thing,
@@ -1955,7 +1995,9 @@ struct gps_data_t {
 #define ERROR_SET	(1llu<<31)
 #define TOFF_SET	(1llu<<32)	/* not yet used */
 #define PPS_SET 	(1llu<<33)
-#define SET_HIGH_BIT	34
+#define NAVDATA_SET     (1llu<<34)
+#define OSCILLATOR_SET	(1llu<<35)
+#define SET_HIGH_BIT	36
     timestamp_t online;		/* NZ if GPS is on line, 0 if not.
 				 *
 				 * Note: gpsd clears this time when sentences
@@ -1979,7 +2021,8 @@ struct gps_data_t {
     /* GPS status -- always valid */
     int    status;		/* Do we have a fix? */
 #define STATUS_NO_FIX	0	/* no */
-#define STATUS_FIX	1	/* yes */
+#define STATUS_FIX	1	/* yes, without DGPS */
+#define STATUS_DGPS_FIX	2	/* yes, with DGPS */
 
     /* precision of fix -- valid if satellites_used > 0 */
     int satellites_used;	/* Number of satellites used in solution */
@@ -2004,7 +2047,7 @@ struct gps_data_t {
     } devices;
 
     /* pack things never reported together to reduce structure size */
-#define UNION_SET	(RTCM2_SET|RTCM3_SET|SUBFRAME_SET|AIS_SET|ATTITUDE_SET|GST_SET|VERSION_SET|LOGMESSAGE_SET|ERROR_SET|TOFF_SET|PPS_SET)
+#define UNION_SET	(RTCM2_SET|RTCM3_SET|SUBFRAME_SET|AIS_SET|ATTITUDE_SET|GST_SET|OSCILLATOR_SET|VERSION_SET|LOGMESSAGE_SET|ERROR_SET|TOFF_SET|PPS_SET)
     union {
 	/* unusual forms of sensor data that might come up the pipe */
 	struct rtcm2_t	rtcm2;
@@ -2012,58 +2055,61 @@ struct gps_data_t {
 	struct subframe_t subframe;
 	struct ais_t ais;
 	struct attitude_t attitude;
+        struct navdata_t navdata;
 	struct rawdata_t raw;
 	struct gst_t gst;
+	struct oscillator_t osc;
 	/* "artificial" structures for various protocol responses */
 	struct version_t version;
 	char error[256];
 	struct timedelta_t toff;
 	struct timedelta_t pps;
     };
+    /* FIXME! next lib rev need to add a place to put PPS precision */
 
     /* Private data - client code must not set this */
     void *privdata;
 };
 
-extern int gps_open(/*@null@*/const char *, /*@null@*/const char *,
-		      /*@out@*/struct gps_data_t *);
+extern int gps_open(const char *, const char *,
+		      struct gps_data_t *);
 extern int gps_close(struct gps_data_t *);
 extern int gps_send(struct gps_data_t *, const char *, ... );
-extern int gps_read(/*@out@*/struct gps_data_t *);
+extern int gps_read(struct gps_data_t *);
 extern int gps_unpack(char *, struct gps_data_t *);
 extern bool gps_waiting(const struct gps_data_t *, int);
-extern int gps_stream(struct gps_data_t *, unsigned int, /*@null@*/void *);
+extern int gps_stream(struct gps_data_t *, unsigned int, void *);
 extern int gps_mainloop(struct gps_data_t *, int,
 			void (*)(struct gps_data_t *));
-extern const char /*@null observer@*/ *gps_data(const struct gps_data_t *);
-extern const char /*@observer@*/ *gps_errstr(const int);
+extern const char *gps_data(const struct gps_data_t *);
+extern const char *gps_errstr(const int);
 
 int json_toff_read(const char *buf, struct gps_data_t *,
-		  /*@null@*/ const char **);
+		  const char **);
 int json_pps_read(const char *buf, struct gps_data_t *,
-		  /*@null@*/ const char **);
+		  const char **);
+int json_oscillator_read(const char *buf, struct gps_data_t *,
+			 const char **);
 
 /* dependencies on struct gpsdata_t end here */
 
 extern void libgps_trace(int errlevel, const char *, ...);
 
-extern void gps_clear_fix(/*@ out @*/struct gps_fix_t *);
-extern void gps_clear_dop( /*@out@*/ struct dop_t *);
-extern void gps_merge_fix(/*@ out @*/struct gps_fix_t *,
-			  gps_mask_t,
-			  /*@ in @*/struct gps_fix_t *);
+extern void gps_clear_fix(struct gps_fix_t *);
+extern void gps_clear_dop( struct dop_t *);
+extern void gps_merge_fix(struct gps_fix_t *, gps_mask_t, struct gps_fix_t *);
 extern void gps_enable_debug(int, FILE *);
-extern /*@observer@*/const char *gps_maskdump(gps_mask_t);
+extern const char *gps_maskdump(gps_mask_t);
 
 extern double safe_atof(const char *);
 extern time_t mkgmtime(register struct tm *);
 extern timestamp_t timestamp(void);
 extern timestamp_t iso8601_to_unix(char *);
-extern /*@observer@*/char *unix_to_iso8601(timestamp_t t, /*@ out @*/char[], size_t len);
+extern char *unix_to_iso8601(timestamp_t t, char[], size_t len);
 extern double earth_distance(double, double, double, double);
 extern double earth_distance_and_bearings(double, double, double, double,
-					  /*@null@*//*@out@*/double *,
-					  /*@null@*//*@out@*/double *);
+					  double *,
+					  double *);
 extern double wgs84_separation(double, double);
 
 /* some multipliers for interpreting GPS output */
