@@ -31,10 +31,12 @@ identified by the string naming its slave device.
 
 TestSession also has methods to start and end client sessions.  Daemon
 responses to a client are fed to a hook function which, by default,
-discards them.  You can change the hook to sys.stdout.write() to dump
-responses to standard output (this is what the gpsfake executable
-does) or do something more exotic. A client session is identified by a
-small integer that counts the number of client session starts.
+discards them.  Note that this data is 'bytes' to accommodate possible
+binary data in Python 3; use polystr() if you need a str.  You can
+change the hook to misc.get_bytes_stream(sys.stdout).write to dump
+responses to standard output (this is what the gpsfake executable does)
+or do something more exotic. A client session is identified by a small
+integer that counts the number of client session starts.
 
 There are a couple of convenience methods.  TestSession.wait() does nothing,
 allowing a specified number of seconds to elapse.  TestSession.send()
@@ -67,10 +69,25 @@ To allow for adding and removing clients while the test is running,
 run in threaded mode by calling the start() method.  This simply calls
 the run method in a subthread, with locking of critical regions.
 """
-import os, sys, time, signal, pty, termios # fcntl, array, struct
-import exceptions, threading, socket, select
+# This code runs compatibly under Python 2 and 3.x for x >= 2.
+# Preserve this property!
+from __future__ import absolute_import, print_function, division
+
+import os
+import pty
+import select
+import signal
+import socket
+import stat
+import subprocess
+import sys
+import termios  # fcntl, array, struct
+import threading
+import time
+
 import gps
-import packet as sniffer
+from gps import polybytes
+from . import packet as sniffer
 
 # The magic number below has to be derived from observation.  If
 # it's too high you'll slow the tests down a lot.  If it's too low
@@ -95,95 +112,109 @@ elif sys.platform.startswith("darwin"):
 else:
     WRITE_PAD = 0.004
 
-# Make it easier to test pad values
-if os.getenv("WRITE_PAD"):
-    WRITE_PAD = eval(os.getenv("WRITE_PAD"))
-
 # Additional delays in slow mode
 WRITE_PAD_SLOWDOWN = 0.01
 
 # If a test takes longer than this, we deem it to have timed out
 TEST_TIMEOUT = 60
 
-class TestLoadError(exceptions.Exception):
+
+def GetDelay(slow=False):
+    "Get appropriate per-line delay."
+    delay = WRITE_PAD
+    # Make it easier to test pad values
+    if os.getenv("WRITE_PAD"):
+        delay = eval(os.getenv("WRITE_PAD"))
+    if slow:
+        delay += WRITE_PAD_SLOWDOWN
+    return delay
+
+
+class TestError(BaseException):
     def __init__(self, msg):
-        exceptions.Exception.__init__(self)
+        super(TestError, self).__init__()
         self.msg = msg
 
-class TestLoad:
+
+class TestLoadError(TestError):
+    pass
+
+
+class TestLoad(object):
     "Digest a logfile into a list of sentences we can cycle through."
+
     def __init__(self, logfp, predump=False, slow=False, oneshot=False):
-        self.sentences = []	# This is the interesting part
-        if type(logfp) == type(""):
-            logfp = open(logfp, "r")
+        self.sentences = []  # This is the interesting part
+        if isinstance(logfp, str):
+            logfp = open(logfp, "rb")
         self.name = logfp.name
         self.logfp = logfp
         self.predump = predump
         self.type = None
         self.sourcetype = "pty"
         self.serial = None
-        self.delay = WRITE_PAD
-        if slow:
-            self.delay += WRITE_PAD_SLOWDOWN
+        self.delay = GetDelay(slow)
         self.delimiter = None
         # Stash away a copy in case we need to resplit
         text = logfp.read()
-        logfp = open(logfp.name)
+        logfp = open(logfp.name, 'rb')
         # Grab the packets in the normal way
         getter = sniffer.new()
-        #gps.packet.register_report(reporter)
+        # gps.packet.register_report(reporter)
         type_latch = None
         commentlen = 0
         while True:
+            # Note that packet data is bytes rather than str
             (plen, ptype, packet, _counter) = getter.get(logfp.fileno())
             if plen <= 0:
                 break
             elif ptype == sniffer.COMMENT_PACKET:
                 commentlen += len(packet)
                 # Some comments are magic
-                if "Serial:" in packet:
+                if b"Serial:" in packet:
                     # Change serial parameters
                     packet = packet[1:].strip()
                     try:
                         (_xx, baud, params) = packet.split()
                         baud = int(baud)
-                        if params[0] in ('7', '8'):
+                        if params[0] in (b'7', b'8'):
                             databits = int(params[0])
                         else:
                             raise ValueError
-                        if params[1] in ('N', 'O', 'E'):
+                        if params[1] in (b'N', b'O', b'E'):
                             parity = params[1]
                         else:
                             raise ValueError
-                        if params[2] in ('1', '2'):
+                        if params[2] in (b'1', b'2'):
                             stopbits = int(params[2])
                         else:
                             raise ValueError
                     except (ValueError, IndexError):
-                        raise TestLoadError("bad serial-parameter spec in %s"%\
+                        raise TestLoadError("bad serial-parameter spec in %s" %
                                             self.name)
                     self.serial = (baud, databits, parity, stopbits)
-                elif "Transport: UDP" in packet:
+                elif b"Transport: UDP" in packet:
                     self.sourcetype = "UDP"
-                elif "Transport: TCP" in packet:
+                elif b"Transport: TCP" in packet:
                     self.sourcetype = "TCP"
-                elif "Delay-Cookie:" in packet:
-                    if packet.startswith("#"):
+                elif b"Delay-Cookie:" in packet:
+                    if packet.startswith(b"#"):
                         packet = packet[1:]
                     try:
-                        (_dummy, self.delimiter, delay) = packet.strip().split()
+                        (_dummy, self.delimiter, delay) = \
+                            packet.strip().split()
                         self.delay = float(delay)
                     except ValueError:
-                        raise TestLoadError("bad Delay-Cookie line in %s"%\
+                        raise TestLoadError("bad Delay-Cookie line in %s" %
                                             self.name)
                     self.resplit = True
             else:
                 if type_latch is None:
                     type_latch = ptype
                 if self.predump:
-                    print repr(packet)
+                    print(repr(packet))
                 if not packet:
-                    raise TestLoadError("zero-length packet from %s"%\
+                    raise TestLoadError("zero-length packet from %s" %
                                         self.name)
                 self.sentences.append(packet)
         # Look at the first packet to grok the GPS type
@@ -197,30 +228,32 @@ class TestLoad:
             self.sentences = text[commentlen:].split(self.delimiter)
         # Do we want single-shot operation?
         if oneshot:
-            self.sentences.append("# EOF\n")
+            self.sentences.append(b"# EOF\n")
 
-class PacketError(exceptions.Exception):
-    def __init__(self, msg):
-        exceptions.Exception.__init__(self)
-        self.msg = msg
 
-class FakeGPS:
+class PacketError(TestError):
+    pass
+
+
+class FakeGPS(object):
     def __init__(self, testload, progress=None):
         self.testload = testload
         self.progress = progress
         self.go_predicate = lambda: True
         self.readers = 0
         self.index = 0
-        self.progress("gpsfake: %s provides %d sentences\n" % (self.testload.name, len(self.testload.sentences)))
+        self.progress("gpsfake: %s provides %d sentences\n"
+                      % (self.testload.name, len(self.testload.sentences)))
 
     def write(self, line):
         "Throw an error if this superclass is ever instantiated."
-        raise ValueError, line
+        raise ValueError(line)
 
     def feed(self):
         "Feed a line from the contents of the GPS log to the daemon."
-        line = self.testload.sentences[self.index % len(self.testload.sentences)]
-        if "%Delay:" in line:
+        line = self.testload.sentences[self.index
+                                       % len(self.testload.sentences)]
+        if b"%Delay:" in line:
             # Delay specified number of seconds
             delay = line.split()[1]
             time.sleep(int(delay))
@@ -229,12 +262,14 @@ class FakeGPS:
         time.sleep(self.testload.delay)
         self.index += 1
 
+
 class FakePTY(FakeGPS):
     "A FakePTY is a pty with a test log ready to be cycled to it."
+
     def __init__(self, testload,
                  speed=4800, databits=8, parity='N', stopbits=1,
                  progress=None):
-        FakeGPS.__init__(self, testload, progress)
+        super(FakePTY, self).__init__(testload, progress)
         # Allow Serial: header to be overridden by explicit speed.
         if self.testload.serial:
             (speed, databits, parity, stopbits) = self.testload.serial
@@ -262,13 +297,17 @@ class FakePTY(FakeGPS):
         }
         (self.fd, self.slave_fd) = pty.openpty()
         self.byname = os.ttyname(self.slave_fd)
-        (iflag, oflag, cflag, lflag, ispeed, ospeed, cc) = termios.tcgetattr(self.slave_fd)
+        os.chmod(self.byname, stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP
+                 | stat.S_IWGRP | stat.S_IROTH | stat.S_IWOTH)
+        (iflag, oflag, cflag, lflag, ispeed, ospeed, cc) = termios.tcgetattr(
+            self.slave_fd)
         cc[termios.VMIN] = 1
         cflag &= ~(termios.PARENB | termios.PARODD | termios.CRTSCTS)
         cflag |= termios.CREAD | termios.CLOCAL
         iflag = oflag = lflag = 0
-        iflag &=~ (termios.PARMRK | termios.INPCK)
-        cflag &=~ (termios.CSIZE | termios.CSTOPB | termios.PARENB | termios.PARODD)
+        iflag &= ~ (termios.PARMRK | termios.INPCK)
+        cflag &= ~ (termios.CSIZE | termios.CSTOPB | termios.PARENB
+                    | termios.PARODD)
         if databits == 7:
             cflag |= termios.CS7
         else:
@@ -285,62 +324,87 @@ class FakePTY(FakeGPS):
         ispeed = ospeed = baudrates[speed]
         try:
             termios.tcsetattr(self.slave_fd, termios.TCSANOW,
-                          [iflag, oflag, cflag, lflag, ispeed, ospeed, cc])
+                              [iflag, oflag, cflag, lflag, ispeed, ospeed, cc])
         except termios.error:
-            raise TestLoadError("error attempting to set serial mode to %s %s%s%s" \
+            raise TestLoadError("error attempting to set serial mode to %s "
+                                " %s%s%s"
                                 % (speed, databits, parity, stopbits))
+
     def read(self):
         "Discard control strings written by gpsd."
         # A tcflush implementation works on Linux but fails on OpenBSD 4.
         termios.tcflush(self.fd, termios.TCIFLUSH)
         # Alas, the FIONREAD version also works on Linux and fails on OpenBSD.
-        #try:
+        # try:
         #    buf = array.array('i', [0])
         #    fcntl.ioctl(self.master_fd, termios.FIONREAD, buf, True)
         #    n = struct.unpack('i', buf)[0]
         #    os.read(self.master_fd, n)
-        #except IOError:
+        # except IOError:
         #    pass
 
     def write(self, line):
-        self.progress("gpsfake: %s writes %d=%s\n" % (self.testload.name, len(line), repr(line)))
+        self.progress("gpsfake: %s writes %d=%s\n"
+                      % (self.testload.name, len(line), repr(line)))
         os.write(self.fd, line)
 
     def drain(self):
         "Wait for the associated device to drain (e.g. before closing)."
         termios.tcdrain(self.fd)
 
-def cleansocket(host, port):
+
+def cleansocket(host, port, socktype=socket.SOCK_STREAM):
     "Get a socket that we can re-use cleanly after it's closed."
-    cs = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    cs = socket.socket(socket.AF_INET, socktype)
     # This magic prevents "Address already in use" errors after
     # we release the socket.
     cs.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
     cs.bind((host, port))
     return cs
 
+
+def freeport(socktype=socket.SOCK_STREAM):
+    """Get a free port number for the given connection type.
+
+    This lets the OS assign a unique port, and then assumes
+    that it will become available for reuse once the socket
+    is closed, and remain so long enough for the real use.
+    """
+    s = cleansocket("127.0.0.1", 0, socktype)
+    port = s.getsockname()[1]
+    s.close()
+    return port
+
+
 class FakeTCP(FakeGPS):
     "A TCP serverlet with a test log ready to be cycled to it."
+
     def __init__(self, testload,
                  host, port,
                  progress=None):
-        FakeGPS.__init__(self, testload, progress)
+        super(FakeTCP, self).__init__(testload, progress)
         self.host = host
-        self.port = int(port)
-        self.byname = "tcp://" + host + ":" + str(port)
-        self.dispatcher = cleansocket(self.host, self.port)
+        self.dispatcher = cleansocket(self.host, int(port))
+        # Get actual assigned port
+        self.port = self.dispatcher.getsockname()[1]
+        self.byname = "tcp://" + host + ":" + str(self.port)
         self.dispatcher.listen(5)
         self.readables = [self.dispatcher]
 
     def read(self):
         "Handle connection requests and data."
-        readable, _writable, _errored = select.select(self.readables, [], [], 0)
+        readable, _writable, _errored = select.select(self.readables, [], [],
+                                                      0)
         for s in readable:
-            if s == self.dispatcher:	# Connection request
+            if s == self.dispatcher:  # Connection request
                 client_socket, _address = s.accept()
                 self.readables = [client_socket]
-                self.dispatcher.close()
-            else:			# Incoming data
+                # Depending on timing, gpsd may try to reconnect between the
+                # end of the log data and the remove_device.  With no listener,
+                # this results in spurious error messages.  Keeping the
+                # listener around avoids this.  It will eventually be closed
+                # by the Python object cleanup. self.dispatcher.close()
+            else:  # Incoming data
                 data = s.recv(1024)
                 if not data:
                     s.close()
@@ -348,7 +412,8 @@ class FakeTCP(FakeGPS):
 
     def write(self, line):
         "Send the next log packet to everybody connected."
-        self.progress("gpsfake: %s writes %d=%s\n" % (self.testload.name, len(line), repr(line)))
+        self.progress("gpsfake: %s writes %d=%s\n"
+                      % (self.testload.name, len(line), repr(line)))
         for s in self.readables:
             if s != self.dispatcher:
                 s.send(line)
@@ -359,12 +424,14 @@ class FakeTCP(FakeGPS):
             if s != self.dispatcher:
                 s.shutdown(socket.SHUT_RDWR)
 
+
 class FakeUDP(FakeGPS):
     "A UDP broadcaster with a test log ready to be cycled to it."
+
     def __init__(self, testload,
                  ipaddr, port,
                  progress=None):
-        FakeGPS.__init__(self, testload, progress)
+        super(FakeUDP, self).__init__(testload, progress)
         self.ipaddr = ipaddr
         self.port = port
         self.byname = "udp://" + ipaddr + ":" + str(port)
@@ -375,83 +442,125 @@ class FakeUDP(FakeGPS):
         pass
 
     def write(self, line):
-        self.progress("gpsfake: %s writes %d=%s\n" % (self.testload.name, len(line), repr(line)))
+        self.progress("gpsfake: %s writes %d=%s\n"
+                      % (self.testload.name, len(line), repr(line)))
         self.sock.sendto(line, (self.ipaddr, int(self.port)))
 
     def drain(self):
         "Wait for the associated device to drain (e.g. before closing)."
-        pass	# shutdown() fails on UDP
+        pass  # shutdown() fails on UDP
 
-class DaemonError(exceptions.Exception):
-    def __init__(self, msg):
-        exceptions.Exception.__init__(self)
-        self.msg = msg
+
+class SubprogramError(TestError):
     def __str__(self):
         return repr(self.msg)
 
-class DaemonInstance:
+
+class SubprogramInstance(object):
+    "Class for generic subprogram."
+    ERROR = SubprogramError
+
+    def __init__(self):
+        self.spawncmd = None
+        self.process = None
+        self.returncode = None
+        self.env = None
+
+    def spawn_sub(self, program, options, background=False, prefix="",
+                  env=None):
+        "Spawn a subprogram instance."
+        spawncmd = None
+
+        # Look for program in GPSD_HOME env variable
+        if os.environ.get('GPSD_HOME'):
+            for path in os.environ['GPSD_HOME'].split(':'):
+                _spawncmd = "%s/%s" % (path, program)
+                if os.path.isfile(_spawncmd) and os.access(_spawncmd, os.X_OK):
+                    spawncmd = _spawncmd
+                    break
+
+        # if we could not find it yet try PATH env variable for it
+        if not spawncmd:
+            if '/usr/sbin' not in os.environ['PATH']:
+                os.environ['PATH'] = os.environ['PATH'] + ":/usr/sbin"
+            for path in os.environ['PATH'].split(':'):
+                _spawncmd = "%s/%s" % (path, program)
+                if os.path.isfile(_spawncmd) and os.access(_spawncmd, os.X_OK):
+                    spawncmd = _spawncmd
+                    break
+
+        if not spawncmd:
+            raise self.ERROR("Cannot execute %s: executable not found. "
+                             "Set GPSD_HOME env variable" % program)
+        self.spawncmd = [spawncmd] + options.split()
+        if prefix:
+            self.spawncmd = prefix.split() + self.spawncmd
+        if env:
+            self.env = os.environ.copy()
+            self.env.update(env)
+        self.process = subprocess.Popen(self.spawncmd, env=self.env)
+        if not background:
+            self.returncode = status = self.process.wait()
+            if os.WIFSIGNALED(status) or os.WEXITSTATUS(status):
+                raise self.ERROR("%s exited with status %d"
+                                 % (program, status))
+
+    def is_alive(self):
+        "Is the program still alive?"
+        if not self.process:
+            return False
+        self.returncode = self.process.poll()
+        if self.returncode is None:
+            return True
+        self.process = None
+        return False
+
+    def kill(self):
+        "Kill the program instance."
+        while self.is_alive():
+            try:  # terminate() may fail if already killed
+                self.process.terminate()
+            except OSError:
+                continue
+            time.sleep(0.01)
+
+
+class DaemonError(SubprogramError):
+    pass
+
+
+class DaemonInstance(SubprogramInstance):
     "Control a gpsd instance."
+    ERROR = DaemonError
+
     def __init__(self, control_socket=None):
-        self.sockfile = None
-        self.pid = None
-        self.tmpdir = os.environ.get('TMPDIR', '/tmp')
+        super(DaemonInstance, self).__init__()
         if control_socket:
             self.control_socket = control_socket
         else:
-            self.control_socket = "%s/gpsfake-%d.sock" % (self.tmpdir, os.getpid())
-        self.pidfile = "%s/gpsfake-%d.pid" % (self.tmpdir, os.getpid())
+            tmpdir = os.environ.get('TMPDIR', '/tmp')
+            self.control_socket = "%s/gpsfake-%d.sock" % (tmpdir, os.getpid())
+
     def spawn(self, options, port, background=False, prefix=""):
         "Spawn a daemon instance."
-        self.spawncmd = None
-
-	# Look for gpsd in GPSD_HOME env variable
-        if os.environ.get('GPSD_HOME'):
-            for path in os.environ['GPSD_HOME'].split(':'):
-                _spawncmd = "%s/gpsd" % path
-                if os.path.isfile(_spawncmd) and os.access(_spawncmd, os.X_OK):
-                    self.spawncmd = _spawncmd
-                    break
-
-	# if we could not find it yet try PATH env variable for it
-        if not self.spawncmd:
-            if not '/usr/sbin' in os.environ['PATH']:
-                os.environ['PATH']=os.environ['PATH'] + ":/usr/sbin"
-            for path in os.environ['PATH'].split(':'):
-                _spawncmd = "%s/gpsd" % path
-                if os.path.isfile(_spawncmd) and os.access(_spawncmd, os.X_OK):
-                    self.spawncmd = _spawncmd
-                    break
-
-        if not self.spawncmd:
-            raise DaemonError("Cannot execute gpsd: executable not found. Set GPSD_HOME env variable")
         # The -b option to suppress hanging on probe returns is needed to cope
-        # with OpenBSD (and possibly other non-Linux systems) that don't support
-        # anything we can use to implement the FakeGPS.read() method
-        self.spawncmd += " -b -N -S %s -F %s -P %s %s" % (port, self.control_socket, self.pidfile, options)
-        if prefix:
-            self.spawncmd = prefix + " " + self.spawncmd.strip()
-        if background:
-            self.spawncmd += " &"
-        status = os.system(self.spawncmd)
-        if os.WIFSIGNALED(status) or os.WEXITSTATUS(status):
-            raise DaemonError("daemon exited with status %d" % status)
-    def wait_pid(self):
-        "Wait for the daemon, get its PID and a control-socket connection."
-        while True:
-            try:
-                fp = open(self.pidfile)
-            except IOError:
-                time.sleep(0.1)
-                continue
-            try:
-                fp.seek(0)
-                pidstr = fp.read()
-                self.pid = int(pidstr)
-            except ValueError:
-                time.sleep(0.5)
-                continue	# Avoid race condition -- PID not yet written
-            fp.close()
-            break
+        # with OpenBSD (and possibly other non-Linux systems) that don't
+        # support anything we can use to implement the FakeGPS.read() method
+        opts = (" -b -N -S %s -F %s %s"
+                % (port, self.control_socket, options))
+        # Derive a unique SHM key from the port # to avoid collisions.
+        # Use 'Gp' as the prefix to avoid colliding with 'GPSD'.
+        shmkey = '0x4770%.04X' % int(port)
+        env = {'GPSD_SHM_KEY': shmkey}
+        self.spawn_sub('gpsd', opts, background, prefix, env)
+
+    def wait_ready(self):
+        "Wait for the daemon to create the control socket."
+        while self.is_alive():
+            if os.path.exists(self.control_socket):
+                return
+            time.sleep(0.1)
+
     def __get_control_socket(self):
         # Now we know it's running, get a connection to the control socket.
         if not os.path.exists(self.control_socket):
@@ -464,49 +573,34 @@ class DaemonInstance:
                 self.sock.close()
             self.sock = None
         return self.sock
-    def is_alive(self):
-        "Is the daemon still alive?"
-        try:
-            os.kill(self.pid, 0)
-            return True
-        except OSError:
-            return False
+
     def add_device(self, path):
         "Add a device to the daemon's internal search list."
         if self.__get_control_socket():
-            self.sock.sendall("+%s\r\n\x00" % path)
+            self.sock.sendall(polybytes("+%s\r\n\x00" % path))
             self.sock.recv(12)
             self.sock.close()
+
     def remove_device(self, path):
         "Remove a device from the daemon's internal search list."
         if self.__get_control_socket():
-            self.sock.sendall("-%s\r\n\x00" % path)
+            self.sock.sendall(polybytes("-%s\r\n\x00" % path))
             self.sock.recv(12)
             self.sock.close()
-    def kill(self):
-        "Kill the daemon instance."
-        if self.pid:
-            try:
-                os.kill(self.pid, signal.SIGTERM)
-                # Raises an OSError for ESRCH when we've killed it.
-                while True:
-                    os.kill(self.pid, signal.SIGTERM)
-                    time.sleep(0.01)
-            except OSError:
-                pass
-            self.pid = None
 
-class TestSessionError(exceptions.Exception):
-    def __init__(self, msg):
-        exceptions.Exception.__init__(self)
-        self.msg = msg
 
-class TestSession:
+class TestSessionError(TestError):
+    # why does testSessionError() return pass? "
+    pass
+
+
+class TestSession(object):
     "Manage a session including a daemon with fake GPSes and clients."
-    def __init__(self, prefix=None, port=None, options=None, verbose=0, predump=False, udp=False, tcp=False, slow=False):
+
+    def __init__(self, prefix=None, port=None, options=None, verbose=0,
+                 predump=False, udp=False, tcp=False, slow=False):
         "Initialize the test session by launching the daemon."
         self.prefix = prefix
-        self.port = port
         self.options = options
         self.verbose = verbose
         self.predump = predump
@@ -520,45 +614,44 @@ class TestSession:
         self.writers = 0
         self.runqueue = []
         self.index = 0
-        self.baseport = 49194	# In the IANA orivate port range
         if port:
             self.port = port
         else:
-            # Magic way to get a socket with an unused port number
-            s = cleansocket("localhost", 0)
-            self.port = s.getsockname()[1]
-            s.close()
+            self.port = freeport()
         self.progress = lambda x: None
         self.reporter = lambda x: None
         self.default_predicate = None
         self.fd_set = []
         self.threadlock = None
+
     def spawn(self):
         for sig in (signal.SIGQUIT, signal.SIGINT, signal.SIGTERM):
             signal.signal(sig, lambda unused, dummy: self.cleanup())
-        self.daemon.spawn(background=True, prefix=self.prefix, port=self.port, options=self.options)
-        self.daemon.wait_pid()
+        self.daemon.spawn(background=True, prefix=self.prefix, port=self.port,
+                          options=self.options)
+        self.daemon.wait_ready()
+
     def set_predicate(self, pred):
         "Set a default go predicate for the session."
         self.default_predicate = pred
+
     def gps_add(self, logfile, speed=19200, pred=None, oneshot=False):
         "Add a simulated GPS being fed by the specified logfile."
         self.progress("gpsfake: gps_add(%s, %d)\n" % (logfile, speed))
         if logfile not in self.fakegpslist:
-            testload = TestLoad(logfile, predump=self.predump, slow=self.slow, oneshot=oneshot)
+            testload = TestLoad(logfile, predump=self.predump, slow=self.slow,
+                                oneshot=oneshot)
             if testload.sourcetype == "UDP" or self.udp:
                 newgps = FakeUDP(testload, ipaddr="127.0.0.1",
-                                 port=self.baseport,
+                                 port=freeport(socket.SOCK_DGRAM),
                                  progress=self.progress)
-                self.baseport += 1
             elif testload.sourcetype == "TCP" or self.tcp:
-                newgps = FakeTCP(testload, host="127.0.0.1",
-                                 port=self.baseport,
+                # Let OS assign the port
+                newgps = FakeTCP(testload, host="127.0.0.1", port=0,
                                  progress=self.progress)
-                self.baseport += 1
             else:
                 newgps = FakePTY(testload, speed=speed,
-                                   progress=self.progress)
+                                 progress=self.progress)
             if pred:
                 newgps.go_predicate = pred
             elif self.default_predicate:
@@ -568,6 +661,7 @@ class TestSession:
             newgps.exhausted = 0
         self.daemon.add_device(newgps.byname)
         return newgps.byname
+
     def gps_remove(self, name):
         "Remove a simulated GPS from the daemon's search list."
         self.progress("gpsfake: gps_remove(%s)\n" % name)
@@ -575,17 +669,25 @@ class TestSession:
         self.remove(self.fakegpslist[name])
         self.daemon.remove_device(name)
         del self.fakegpslist[name]
+
     def client_add(self, commands):
         "Initiate a client session and force connection to a fake GPS."
         self.progress("gpsfake: client_add()\n")
-        newclient = gps.gps(port=self.port, verbose=self.verbose)
+        try:
+            newclient = gps.gps(port=self.port, verbose=self.verbose)
+        except socket.error:
+            if not self.daemon.is_alive():
+                raise TestSessionError("daemon died")
+            raise
         self.append(newclient)
         newclient.id = self.client_id + 1
         self.client_id += 1
-        self.progress("gpsfake: client %d has %s\n" % (self.client_id,newclient.device))
+        self.progress("gpsfake: client %d has %s\n"
+                      % (self.client_id, newclient.device))
         if commands:
             self.initialize(newclient, commands)
         return self.client_id
+
     def client_remove(self, cid):
         "Terminate a client session."
         self.progress("gpsfake: client_remove(%d)\n" % cid)
@@ -594,21 +696,24 @@ class TestSession:
                 self.remove(obj)
                 return True
         return False
+
     def wait(self, seconds):
         "Wait, doing nothing."
         self.progress("gpsfake: wait(%d)\n" % seconds)
         time.sleep(seconds)
+
     def gather(self, seconds):
         "Wait, doing nothing but watching for sentences."
         self.progress("gpsfake: gather(%d)\n" % seconds)
-        #mark = time.time()
         time.sleep(seconds)
+
     def cleanup(self):
         "We're done, kill the daemon."
         self.progress("gpsfake: cleanup()\n")
         if self.daemon:
             self.daemon.kill()
             self.daemon = None
+
     def run(self):
         "Run the tests."
         try:
@@ -623,13 +728,18 @@ class TestSession:
                 had_output = False
                 chosen = self.choose()
                 if isinstance(chosen, FakeGPS):
-                    if chosen.exhausted and (time.time() - chosen.exhausted > TEST_TIMEOUT) and chosen.byname in self.fakegpslist:
-                        sys.stderr.write("Test timed out: increase WRITE_PAD = %s\n" % WRITE_PAD)
-                        raise SystemExit, 1
+                    if ((chosen.exhausted
+                         and (time.time() - chosen.exhausted > TEST_TIMEOUT)
+                         and chosen.byname in self.fakegpslist)):
+                        sys.stderr.write(
+                            "Test timed out: increase WRITE_PAD = %s\n"
+                            % GetDelay(self.slow))
+                        raise SystemExit(1)
                     elif not chosen.go_predicate(chosen.index, chosen):
                         if chosen.exhausted == 0:
                             chosen.exhausted = time.time()
-                            self.progress("gpsfake: GPS %s ran out of input\n" % chosen.byname)
+                            self.progress("gpsfake: GPS %s ran out of input\n"
+                                          % chosen.byname)
                     else:
                         chosen.feed()
                 elif isinstance(chosen, gps.gps):
@@ -638,12 +748,17 @@ class TestSession:
                         chosen.enqueued = ""
                     while chosen.waiting():
                         chosen.read()
-                        if chosen.valid & gps.PACKET_SET:
-                            self.reporter(chosen.response)
-                            if chosen.data["class"] == "DEVICE" and chosen.data["activated"] == 0 and chosen.data["path"] in self.fakegpslist:
-                                self.gps_remove(chosen.data["path"])
-                                self.progress("gpsfake: GPS %s removed (notification)\n" % chosen.data["path"])
                         had_output = True
+                        if not chosen.valid & gps.PACKET_SET:
+                            continue
+                        self.reporter(chosen.bresponse)
+                        if ((chosen.data["class"] == "DEVICE"
+                             and chosen.data["activated"] == 0
+                             and chosen.data["path"] in self.fakegpslist)):
+                            self.gps_remove(chosen.data["path"])
+                            self.progress(
+                                "gpsfake: GPS %s removed (notification)\n"
+                                % chosen.data["path"])
                 else:
                     raise TestSessionError("test object of unknown type")
                 if not self.writers and not had_output:
@@ -668,6 +783,7 @@ class TestSession:
             self.readers += 1
         if self.threadlock:
             self.threadlock.release()
+
     def remove(self, obj):
         "Remove a producer or consumer from the object list."
         if self.threadlock:
@@ -677,9 +793,10 @@ class TestSession:
             self.writers -= 1
         elif isinstance(obj, gps.gps):
             self.readers -= 1
-        self.index = min(len(self.runqueue)-1, self.index)
+        self.index = min(len(self.runqueue) - 1, self.index)
         if self.threadlock:
             self.threadlock.release()
+
     def choose(self):
         "Atomically get the next object scheduled to do something."
         if self.threadlock:
@@ -690,6 +807,7 @@ class TestSession:
         if self.threadlock:
             self.threadlock.release()
         return self.runqueue[chosen]
+
     def initialize(self, client, commands):
         "Arrange for client to ship specified commands when it goes active."
         client.enqueued = ""
@@ -697,6 +815,7 @@ class TestSession:
             client.send(commands)
         else:
             client.enqueued = commands
+
     def start(self):
         self.threadlock = threading.Lock()
         threading.Thread(target=self.run)
