@@ -2,11 +2,11 @@
  * This file is Copyright (c) 2010 by the GPSD project
  * BSD terms apply: see the file COPYING in the distribution root for details.
  */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdbool.h>
 #include <string.h>
-#include <syslog.h>
 #include <math.h>
 #include <time.h>
 #include <errno.h>
@@ -19,6 +19,7 @@
 #include "gpsd_config.h"
 #include "gpsdclient.h"
 #include "revision.h"
+#include "os_compat.h"
 
 static char *progname;
 static struct fixsource_t source;
@@ -87,20 +88,23 @@ static void print_fix(struct gps_data_t *gpsdata, double time)
 	(void)fprintf(logfile,"    <ele>%f</ele>\n", gpsdata->fix.altitude);
     (void)fprintf(logfile,"    <time>%s</time>\n",
 		 unix_to_iso8601(time, tbuf, sizeof(tbuf)));
-    switch (gpsdata->fix.mode) {
-    case MODE_3D:
-	(void)fprintf(logfile,"    <fix>3d</fix>\n");
-	break;
-    case MODE_2D:
-	(void)fprintf(logfile,"    <fix>2d</fix>\n");
-	break;
-    case MODE_NO_FIX:
-	(void)fprintf(logfile,"    <fix>none</fix>\n");
-	break;
-    default:
-	/* don't print anything if no fix indicator */
-	break;
-    }
+    if (gpsdata->status == STATUS_DGPS_FIX)
+	(void)fprintf(logfile,"    <fix>dgps</fix>\n");
+    else
+	switch (gpsdata->fix.mode) {
+	case MODE_3D:
+	    (void)fprintf(logfile,"    <fix>3d</fix>\n");
+	    break;
+	case MODE_2D:
+	    (void)fprintf(logfile,"    <fix>2d</fix>\n");
+	    break;
+	case MODE_NO_FIX:
+	    (void)fprintf(logfile,"    <fix>none</fix>\n");
+	    break;
+	default:
+	    /* don't print anything if no fix indicator */
+	    break;
+	}
 
     if ((gpsdata->fix.mode > MODE_NO_FIX) && (gpsdata->satellites_used > 0))
 	(void)fprintf(logfile,"    <sat>%d</sat>\n", gpsdata->satellites_used);
@@ -177,11 +181,12 @@ static void quit_handler(int signum)
 
 static void usage(void)
 {
-    fprintf(stderr,
-	    "Usage: %s [-V] [-h] [-d] [-i timeout] [-f filename] [-m minmove]\n"
-	    "\t[-e exportmethod] [server[:port:[device]]]\n\n"
-	    "defaults to '%s -i 5 -e %s localhost:2947'\n",
-	    progname, progname, export_default()->name);
+    (void)fprintf(stderr,
+                  "Usage: %s [-V] [-h] [-l] [-d] [-D debuglevel]"
+                  " [-i timeout] [-f filename] [-m minmove]\n"
+                  "\t[-r] [-e exportmethod] [server[:port:[device]]]\n\n"
+                  "defaults to '%s -i 5 -e %s localhost:2947'\n",
+                  progname, progname, export_default()->name);
     exit(EXIT_FAILURE);
 }
 
@@ -189,6 +194,7 @@ int main(int argc, char **argv)
 {
     int ch;
     bool daemonize = false;
+    bool reconnect = false;
     unsigned int flags = WATCH_ENABLE;
     struct exportmethod_t *method = NULL;
 
@@ -201,7 +207,7 @@ int main(int argc, char **argv)
     }
 
     logfile = stdout;
-    while ((ch = getopt(argc, argv, "dD:e:f:hi:lm:V")) != -1) {
+    while ((ch = getopt(argc, argv, "dD:e:f:hi:lm:rV")) != -1) {
 	switch (ch) {
 	case 'd':
 	    openlog(basename(progname), LOG_PID | LOG_PERROR, LOG_DAEMON);
@@ -243,21 +249,23 @@ int main(int argc, char **argv)
                 }
                 fname[s] = '\0';;
                 logfile = fopen(fname, "w");
-                if (logfile == NULL)
+                if (logfile == NULL) {
 		    syslog(LOG_ERR,
 			   "Failed to open %s: %s, logging to stdout.",
 			   fname, strerror(errno));
+		    logfile = stdout;
+		}
 	    bailout:
                 free(fname);
                 break;
             }
-	case 'i':		/* set polling interfal */
+	case 'i':		/* set polling interval */
 	    timeout = (time_t) atoi(optarg);
 	    if (timeout < 1)
 		timeout = 1;
 	    if (timeout >= 3600)
-		fprintf(stderr,
-			"WARNING: track timeout is an hour or more!\n");
+		(void)fprintf(stderr,
+		              "WARNING: track timeout is an hour or more!\n");
 	    break;
 	case 'l':
 	    export_list(stderr);
@@ -265,6 +273,9 @@ int main(int argc, char **argv)
         case 'm':
 	    minmove = (double )atoi(optarg);
 	    break;
+        case 'r':
+	    reconnect = true;
+	    break;
 	case 'V':
 	    (void)fprintf(stderr, "%s: version %s (revision %s)\n",
 			  progname, VERSION, REVISION);
@@ -307,8 +318,8 @@ int main(int argc, char **argv)
     /* might be time to daemonize */
     if (daemonize) {
 	/* not SuS/POSIX portable, but we have our own fallback version */
-	if (daemon(0, 0) != 0)
-	    (void) fprintf(stderr,"demonization failed: %s\n", strerror(errno));
+	if (os_daemon(0, 0) != 0)
+	    (void) fprintf(stderr,"daemonization failed: %s\n", strerror(errno));
     }
 
     //syslog (LOG_INFO, "---------- STARTED ----------");
@@ -325,7 +336,14 @@ int main(int argc, char **argv)
     (void)gps_stream(&gpsdata, flags, source.device);
 
     print_gpx_header();
-    (void)gps_mainloop(&gpsdata, 5000000, conditionally_log_fix);
+
+    while (gps_mainloop(&gpsdata, timeout * 1000000, conditionally_log_fix) < 0 &&
+	   reconnect) {
+	/* avoid busy-calling gps_mainloop() */
+	(void)sleep(timeout);
+	syslog(LOG_INFO, "timeout; about to reconnect");
+    }
+
     print_gpx_footer();
     (void)gps_close(&gpsdata);
 
