@@ -67,10 +67,9 @@ PERMISSIONS
 #include <stdlib.h>
 #include <stdbool.h>
 #include <stdarg.h>
-#include <time.h>               /* for time_t */
 #include <ctype.h>
 
-#include "gpsd_config.h"	/* for strlcpy() prototype */
+#include "os_compat.h"
 #ifdef SOCKET_EXPORT_ENABLE
 #include "json.h"
 
@@ -421,6 +420,7 @@ static int json_internal_read_object(const char *cp,
 	    } else
 		*pval++ = *cp;
 	    break;
+	    /* coverity[unterminated_case] */
 	case post_val:
 	    /*
 	     * We know that cursor points at the first spec matching
