@@ -12,9 +12,7 @@
 #include <stdarg.h>
 #include <stdbool.h>
 #include <assert.h>
-#ifndef S_SPLINT_S
 #include <unistd.h>
-#endif /* S_SPLINT_S */
 
 #include "gpsd.h"
 #include "bits.h"
