@@ -9,15 +9,12 @@
 #include <stdlib.h>
 #include <string.h>
 #include <fcntl.h>
-#ifndef S_SPLINT_S
 #include <netdb.h>
 #include <sys/socket.h>
 #include <unistd.h>
-#endif /* S_SPLINT_S */
 
 #include "gpsd.h"
 
-/*@ -branchstate */
 int dgpsip_open(struct gps_device_t *device, const char *dgpsserver)
 /* open a connection to a DGPSIP server */
 {
@@ -60,8 +57,6 @@ int dgpsip_open(struct gps_device_t *device, const char *dgpsserver)
     return device->gpsdata.gps_fd;
 }
 
-/*@ +branchstate */
-
 void dgpsip_report(struct gps_context_t *context,
 		   struct gps_device_t *gps,
 		   struct gps_device_t *dgpsip)
