@@ -8,10 +8,8 @@
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#ifndef S_SPLINT_S
 #include <sys/socket.h>
 #include <unistd.h>
-#endif /* S_SPLINT_S */
 
 #include "gpsd.h"
 #include "strfuncs.h"
@@ -28,7 +26,6 @@ bool netgnss_uri_check(char *name)
 }
 
 
-/*@ -branchstate */
 int netgnss_uri_open(struct gps_device_t *dev, char *netgnss_service)
 /* open a connection to a DGNSS service */
 {
@@ -51,7 +48,6 @@ int netgnss_uri_open(struct gps_device_t *dev, char *netgnss_service)
     return -1;
 #endif
 }
-/*@ +branchstate */
 
 void netgnss_report(struct gps_context_t *context,
 		    struct gps_device_t *gps, struct gps_device_t *dgnss)
